function fitnessFuc = ObjectiveFunction(phase)

% Two cost functions of maximum likelihood estimations "L" from "Joint estimation of object and aberrations by using phase diversity"
% & "Fast algorithm for phase diversity-based blind deconvolution" are included in this function to estimate the PSF, could try both one by one.

global N_Recon CircleAperture_Recon NumChannel I1_current DiverseSpeckle_I2 DiversePhaseStore regularization weight

IndividualPhase_vector = phase;
IndividualPhase_matrix = reshape(IndividualPhase_vector,N_Recon,N_Recon);
PupilFunctionGuess = CircleAperture_Recon.*exp(1i.*IndividualPhase_matrix); 
ImpulseResponseGuess = ifftshift(ifft2(fftshift(PupilFunctionGuess)));
PSF1_Guess = ImpulseResponseGuess;
PSF1_Guess = abs(PSF1_Guess).^2;
PSF1_Guess = PSF1_Guess.*weight;

PSF1_FFT = fftshift(fft2(fftshift(PSF1_Guess)));
conj_PSF1_FFT = conj(PSF1_FFT);
I1_FFT = fftshift(fft2(fftshift(I1_current)));
conj_I1_FFT = conj(I1_FFT);

% from the paper of "Joint estimation of object and aberrations by using phase diversity"
L_Matrix_firstTerm_numerator = I1_FFT.*conj_PSF1_FFT;
L_Matrix_firstTerm_denominator = abs(PSF1_FFT).^2;
L_Matrix_secondTerm = abs(I1_FFT).^2;

% % from the paper of "Fast algorithm for phase diversity-based blind deconvolution"
% L_Matrix_firstTerm_numerator = conj_I1_FFT.*PSF1_FFT;   
% L_Matrix_firstTerm_denominator = abs(PSF1_FFT).^2;
% L_Matrix_secondTerm = abs(I1_FFT).^2;

for k = 1:NumChannel
    
    DefocusPSF = exp(1i.*DiversePhaseStore{k});    
    DiversePupilFunctionGuess = PupilFunctionGuess.*DefocusPSF;
    DiverseImpulseResponseGuess = ifftshift(ifft2(fftshift(DiversePupilFunctionGuess)));
    PSF2_Guess = DiverseImpulseResponseGuess;
    PSF2_Guess = abs(PSF2_Guess).^2;
    PSF2_Guess = PSF2_Guess.*weight;
    
    PSF2_FFT = fftshift(fft2(fftshift(PSF2_Guess)));
    conj_PSF2_FFT = conj(PSF2_FFT);
    I2_current = DiverseSpeckle_I2{k};
    I2_FFT = fftshift(fft2(fftshift(I2_current)));
    conj_I2_FFT = conj(I2_FFT);
    
    % from the paper of "Joint estimation of object and aberrations by using phase diversity"
    L_Matrix_firstTerm_numerator = L_Matrix_firstTerm_numerator+I2_FFT.*conj_PSF2_FFT;
    L_Matrix_firstTerm_denominator = L_Matrix_firstTerm_denominator+abs(PSF2_FFT).^2;
    L_Matrix_secondTerm = L_Matrix_secondTerm+abs(I2_FFT).^2;
    
%     % from the paper of "Fast algorithm for phase diversity-based blind deconvolution"
%     L_Matrix_firstTerm_numerator = L_Matrix_firstTerm_numerator+conj_I2_FFT.*PSF2_FFT;
%     L_Matrix_firstTerm_denominator = L_Matrix_firstTerm_denominator+abs(PSF2_FFT).^2;
%     L_Matrix_secondTerm = L_Matrix_secondTerm+abs(I2_FFT).^2;
       
end

    % Reference:"Joint estimation of object and aberrations by using phase diversityExellent"
%     L_Matrix = (abs(L_Matrix_firstTerm_numerator).^2)./(L_Matrix_firstTerm_denominator+regularization)-L_Matrix_secondTerm;
    L_Matrix1 = (abs(L_Matrix_firstTerm_numerator).^2)./(L_Matrix_firstTerm_denominator+regularization);  % second term is constant
    L_Matrix2 = L_Matrix_secondTerm;
    L1 = sum(sum(L_Matrix1));
    L2 = sum(sum(L_Matrix2));
    L = L1 - L2;
    L = -L;
    
%     % Reference:"Fast algorithm for phase diversity-based blind deconvolution"
%     L_Matrix = L_Matrix_secondTerm-(abs(L_Matrix_firstTerm_numerator).^2)./(L_Matrix_firstTerm_denominator+regularization);
    
    L_minus = L;
    fitnessFuc = L_minus;

end