clear all;
close all;
clc;
set(0,'DefaultFigureWindowStyle','docked');

global N_Recon CircleAperture_Recon NumChannel I1_current DiverseSpeckle_I2 DiversePhaseStore regularization weight

SpeckleFrame = 19;
NumChannel = SpeckleFrame-1;
Speckles = NumChannel/3;
SpeckleDataMod = cell(SpeckleFrame,1);
SpeckleDataCrop = cell(SpeckleFrame,1);
DiversePhaseStore = cell(SpeckleFrame-1,1);
baseSpeckledimension = 84;
regularization = 1e-5;        % the selection depends, but is important. Too large, too smooth of the reconstruction; 
                              % too small, amplify the error!
DeltaZ = 0.8e-3;           
Z = 12.045e-2;                
lambda = 632.89e-9;           
PupilD = 0.521e-2;             
OTFDimension = 74;            
speckleGrainSize = 2.25;
dx = 6.5e-6;                  
dy = 6.5e-6;                 
PupilDimension = round(OTFDimension/2);    % OTF is the autocorrelation of the pupil function
weight = window2(baseSpeckledimension,baseSpeckledimension,@hann);    % window function
Radius = round(PupilDimension/2);
x = -round(baseSpeckledimension/2):round(baseSpeckledimension/2)-1;
[X,Y] = meshgrid(x,x);
CircleAperture_Recon = sqrt(X.^2+Y.^2)<=Radius;    % estimated pupil


Lx = dx*baseSpeckledimension;
Ly = dy*baseSpeckledimension;
dfx = 1/Lx;            % interval of frequency of X dimension in frequency domain
dfy = 1/Ly;            % interval of frequency of Y dimension in frequency domain
du = lambda*Z*dfx;     % interval of coordinate of X dimension in frequency domain
dv = lambda*Z*dfy;     % interval of coordinate of Y dimension in frequency domain
u = (-round(baseSpeckledimension/2):(round(baseSpeckledimension/2)-1))*du;   
v = (-round(baseSpeckledimension/2):(round(baseSpeckledimension/2)-1))*dv;  
[U,V] = meshgrid(u,v);       % coordinate distribution in frequency domain

for m = 1:NumChannel;
    DiversePhaseStore{m} = ((m*DeltaZ*pi)/lambda/Z/(Z+m*DeltaZ))*(U.^2+V.^2);    
    DiversePhaseStore{m} = atan2(sin(DiversePhaseStore{m}), cos(DiversePhaseStore{m}));    % wrapped phase
    figure(1);
    titlename = strcat('No. ',num2str(m),' Diverse Channel');
    title(titlename);
    subplot(121);imshow(DiversePhaseStore{m},[]);colormap hot;colorbar;
    pause(0.2)
    subplot(122);surf(DiversePhaseStore{m});colormap hot;colorbar;
    pause(0.2);
end


% Filter the speckles
for i = 0:NumChannel
    
    Filename = strcat('Z',num2str(i),'.tiff');
    SpeckleData = mat2gray(double(imread(Filename)));
    SpeckleData = flip(SpeckleData,1);
%     SpeckleData = wiener2(SpeckleData,[2 2]);
    [N_Speckle_1,N_Speckle_2] = size(SpeckleData);
    %% filter begin
    Large_Speckle_intensity_FFT = fftshift(fft2((SpeckleData)));
    LowPass_Filter = zeros(N_Speckle_1,N_Speckle_2);
    Filter_Length = 40;
    LowPass_Filter((floor(N_Speckle_1/2)-Filter_Length):(floor(N_Speckle_1/2)+Filter_Length-1),(floor(N_Speckle_2/2)-Filter_Length):(floor(N_Speckle_2/2)+Filter_Length-1)) = 1;
    Large_Speckle_intensity_FFT = Large_Speckle_intensity_FFT.*LowPass_Filter;
    Large_Speckle_intensity_LowPassVer = abs(ifft2(ifftshift(Large_Speckle_intensity_FFT)));
    Large_Speckle_intensity_New = SpeckleData./Large_Speckle_intensity_LowPassVer;
    h = fspecial('gaussian',[4 4],0.5);
    Large_Speckle_intensity = imfilter(Large_Speckle_intensity_New,h,'replicate');
    Large_Speckle_intensity = Large_Speckle_intensity(1070:1570,950:1450);     % !!!!!  PD_Data10 center part
    SpeckleDataMod{i+1} = Large_Speckle_intensity;
%     SpeckleDataMod{i+1} = SpeckleData;
    
    %% filter end
    figure(2);imshow(Large_Speckle_intensity,[]);colormap hot;colorbar;
    titlename = strcat('No. ',num2str(i+1),' Frame');
    title(titlename);
    pause(0.5);
    
end

% Crop the speckles
[N_Speckle_1,N_Speckle_2] = size(Large_Speckle_intensity);
% [N_Speckle_1,N_Speckle_2] = size(SpeckleData);
crop = N_Speckle_1-(2*baseSpeckledimension);  % select the number of groups for average
SpeckleUse1 = N_Speckle_1-crop;
SpeckleUse2 = N_Speckle_2-crop;

for j = 0:NumChannel

    SpeckleDataCrop{j+1} = SpeckleDataMod{j+1}(round(N_Speckle_1/2)+(-SpeckleUse1/2:(SpeckleUse1/2-1)),round(N_Speckle_2/2)+(-SpeckleUse2/2:(SpeckleUse2/2-1)));
%     SpeckleDataCrop{j+1} = SpeckleDataMod{j+1}(1900:1989,900:989);
    figure(3);imshow(SpeckleDataCrop{j+1},[]);colormap hot;colorbar;
    titlename = strcat('No. ',num2str(j+1),' Frame');
    title(titlename);
    pause(0.5);
  
end

% dividing and storing subimage
overlap_ratio = 0;
num_sqrt_1 = floor(1+(SpeckleUse1-baseSpeckledimension)/((1-overlap_ratio)*baseSpeckledimension));
num_sqrt_2 = floor(1+(SpeckleUse2-baseSpeckledimension)/((1-overlap_ratio)*baseSpeckledimension));
num = num_sqrt_1*num_sqrt_2;
store_subimage = cell(num_sqrt_1,num_sqrt_2,SpeckleFrame);
subimage_FFT_sum = zeros(baseSpeckledimension,baseSpeckledimension);
totalFrame = 0;

for k = 0:NumChannel;
    for p = 1:num_sqrt_1
        for q = 1:num_sqrt_2
            store_subimage{p,q,k+1} = (SpeckleDataCrop{k+1}((uint16((p-1)*(1-overlap_ratio)*baseSpeckledimension+1):uint16(baseSpeckledimension*p-overlap_ratio*(p-1)*baseSpeckledimension)),...
                (uint16((q-1)*(1-overlap_ratio)*baseSpeckledimension+1):uint16(baseSpeckledimension*q-overlap_ratio*(q-1)*baseSpeckledimension))));
            
%             figure(4);imshow(store_subimage{p,q,k+1},[]);colormap hot;colorbar;
%             titlename = strcat('No. ',num2str(k+1),' Frame');
%             title(titlename);
%             pause(0.5);
        end
    end   
end

%% reconstrution with phase diversity

iterationNum = 1200;
N_Recon = baseSpeckledimension;
InitialGuess = zeros(N_Recon,N_Recon);       % zero initial guess should be better than random one at the current stage
InitialGuess = InitialGuess(:);
Obj_reconstruction = zeros(N_Recon,N_Recon);
PSF1Whole_estimation_store = cell(num_sqrt_1,num_sqrt_2);
PSF1_estimation_store = cell(num_sqrt_1,num_sqrt_2);
PSF2Whole_estimation_store = cell(num_sqrt_1,num_sqrt_2);
PSF2_estimation_store = cell(num_sqrt_1,num_sqrt_2);
DiverseSpeckle_I2 = cell(NumChannel,1);
Temp = 0;

for h = 1:num_sqrt_1
    for g = 1:num_sqrt_2
        
        Temp = Temp + 1;
        I1_current = store_subimage{h,g,1}.*weight;
        I1_current_FFT = fftshift(fft2(fftshift(I1_current)));
        
        figure(200);subplot(222);imshow(I1_current,[]);colormap hot;colorbar;axis on;pause(0.5);
        
        for w = 1:NumChannel
            DiverseSpeckle_I2{w} = store_subimage{h,g,w+1}.*weight;
            
            figure(200);subplot(224);imshow(DiverseSpeckle_I2{w},[]);colormap hot;colorbar;axis on;pause(0.5);
            
        end
        
        options = optimoptions(@fminunc,'Algorithm','quasi-newton','MaxIter',iterationNum,'Display','iter','MaxFunEvals',Inf,'TolFun',1e-6,'TolX',1e-6);
        [x,fval] = fminunc(@ObjectiveFunction,InitialGuess,options);
        
        PupilFunction_phase_estimation = x;
        PupilFunction_phase_estimation = reshape(PupilFunction_phase_estimation,N_Recon,N_Recon);
        Pupilphase_estimation_show = atan2(sin(PupilFunction_phase_estimation), cos(PupilFunction_phase_estimation));
        PupilFunction_estimation = CircleAperture_Recon.*exp(1i.*PupilFunction_phase_estimation);  % amplitude|H(u)| = 1;
        ImpulseResponse_estimation = ifftshift(ifft2(fftshift(PupilFunction_estimation)));
        PSF1_Field_estimation = ImpulseResponse_estimation;
        PSF1Whole_estimation = abs(PSF1_Field_estimation).^2;
        PSF1Whole_estimation_store{h,g} = PSF1Whole_estimation;
%         PSF1_estimation = PSF1Whole_estimation;
        PSF1_estimation = PSF1Whole_estimation.*weight;
        PSF1_estimation_store{h,g} = PSF1_estimation;
        PSF1_estimation_FFT = fftshift(fft2(fftshift(PSF1_estimation)));
        conj_PSF1_estimation_FFT = conj(PSF1_estimation_FFT);
        
        ReObj_estimation_FFT_numerator = I1_current_FFT.*conj_PSF1_estimation_FFT;
        ReObj_estimation_FFT_denominator = abs(PSF1_estimation_FFT).^2;
        
        
        for m = 1:NumChannel
            
            I2_current = DiverseSpeckle_I2{m};
            I2_current_FFT = fftshift(fft2(fftshift(I2_current)));
            diversePhase_current = DiversePhaseStore{m};
            
            % find estimated S1(u) and S2(u)
            DiversePupilFunction_estimation = PupilFunction_estimation.*exp(1i.*diversePhase_current);
            DiverseImpulseResponse_estimation = ifftshift(ifft2(fftshift(DiversePupilFunction_estimation)));
            PSF2_Field_estimation = DiverseImpulseResponse_estimation;
            PSF2Whole_estimation = abs(PSF2_Field_estimation).^2;
            PSF2Whole_estimation_store{h,g} = PSF2Whole_estimation;
%             PSF2_estimation = PSF2Whole_estimation;
            PSF2_estimation = PSF2Whole_estimation.*weight;
            PSF2_estimation_store{h,g} = PSF2_estimation;
            PSF2_estimation_FFT = fftshift(fft2(fftshift(PSF2_estimation)));
            conj_PSF2_estimation_FFT = conj(PSF2_estimation_FFT);
            
            ReObj_estimation_FFT_numerator = ReObj_estimation_FFT_numerator+I2_current_FFT.*conj_PSF2_estimation_FFT;
            ReObj_estimation_FFT_denominator = ReObj_estimation_FFT_denominator+abs(PSF2_estimation_FFT).^2;
            
        end
        
        ReObj_estimation_FFT = ReObj_estimation_FFT_numerator./(ReObj_estimation_FFT_denominator+regularization);
        ReObj_estimation = ifftshift(ifft2(fftshift(ReObj_estimation_FFT)));
        Reconstruction = abs(ReObj_estimation);
        Obj_reconstruction = Obj_reconstruction + abs(ReObj_estimation);
        
        figure;
        subplot(221);imshow(abs(ReObj_estimation),[]);colormap hot;colorbar;axis on;
        subplot(222);imshow(abs(ReObj_estimation).^2,[]);colormap hot;colorbar;axis on;
        subplot(223);imshow(PSF1_estimation,[]);colormap hot;colorbar;axis on;
        subplot(224);imshow(angle(PupilFunction_estimation),[]);colormap hot;colorbar;axis on;
        
        fprintf('No.(%d, %d) group of subimages is done!\n\n',h,g);
    end
end

Obj_reconstruction = Obj_reconstruction/num;
figure(200);subplot(223);imshow(Obj_reconstruction,[]);colormap hot;colorbar;axis on;pause(0.5);

